# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, TYPE_CHECKING
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse
from azure.mgmt.core import ARMPipelineClient
from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy

from ._configuration import NeonPostgresMgmtClientConfiguration
from ._serialization import Deserializer, Serializer
from .operations import (
    BranchesOperations,
    ComputesOperations,
    EndpointsOperations,
    ModelsOperations,
    NeonDatabasesOperations,
    NeonRolesOperations,
    Operations,
    OrganizationsOperations,
    ProjectsOperations,
)

if TYPE_CHECKING:
    from azure.core.credentials import TokenCredential


class NeonPostgresMgmtClient:  # pylint: disable=too-many-instance-attributes
    """NeonPostgresMgmtClient.

    :ivar models: ModelsOperations operations
    :vartype models: azure.mgmt.neonpostgres.operations.ModelsOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.neonpostgres.operations.Operations
    :ivar organizations: OrganizationsOperations operations
    :vartype organizations: azure.mgmt.neonpostgres.operations.OrganizationsOperations
    :ivar projects: ProjectsOperations operations
    :vartype projects: azure.mgmt.neonpostgres.operations.ProjectsOperations
    :ivar branches: BranchesOperations operations
    :vartype branches: azure.mgmt.neonpostgres.operations.BranchesOperations
    :ivar computes: ComputesOperations operations
    :vartype computes: azure.mgmt.neonpostgres.operations.ComputesOperations
    :ivar neon_databases: NeonDatabasesOperations operations
    :vartype neon_databases: azure.mgmt.neonpostgres.operations.NeonDatabasesOperations
    :ivar neon_roles: NeonRolesOperations operations
    :vartype neon_roles: azure.mgmt.neonpostgres.operations.NeonRolesOperations
    :ivar endpoints: EndpointsOperations operations
    :vartype endpoints: azure.mgmt.neonpostgres.operations.EndpointsOperations
    :param credential: Credential used to authenticate requests to the service. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
    :type subscription_id: str
    :param base_url: Service host. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: The API version to use for this operation. Default value is "2025-03-01".
     Note that overriding this default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "TokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        self._config = NeonPostgresMgmtClientConfiguration(
            credential=credential, subscription_id=subscription_id, base_url=base_url, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                ARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.models = ModelsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.organizations = OrganizationsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.projects = ProjectsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.branches = BranchesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.computes = ComputesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.neon_databases = NeonDatabasesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.neon_roles = NeonRolesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.endpoints = EndpointsOperations(self._client, self._config, self._serialize, self._deserialize)

    def send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client.send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)
